/*
 * $Id: arp.c,v 1.1.1.1 2004/05/25 17:56:25 mike Exp $
 *
 *  arpsnap
 *  ARP request or reply forgery tool
 *  Released as part of the MXFP Layer 2 Toolkit 
 *  http://www.packetfactory.net/MXFP
 *
 *  Copyright (c) 2004 Mike D. Schiffman  <mike@infonexus.com>
 *  Copyright (c) 2004 Jeremy Rauch       <jrauch@cadre.org>
 *
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <libnet.h> 

void usage (char *);

int
main(int argc, char *argv[])
{
    int c;
    u_int32_t s = 0, d = 0;
    libnet_t *l;
    libnet_ptag_t t;
    char *device = NULL;
    u_int8_t *packet;
    u_int32_t packet_s;
    char errbuf[LIBNET_ERRBUF_SIZE];
    char *srcmac = NULL;
    char *dstmac = NULL;
    int len;
    int ch; 
    int type = ARPOP_REPLY;

    printf("ARPSNAP!\n"); 
    while ((ch = getopt(argc, argv, "rRi:S:D:s:d:h")) != -1)
        switch (ch) {
            case 'R':
                type = ARPOP_REQUEST;
                break;
            case 'r':
                type = ARPOP_REPLY;
                break;
            case 'i':
                device = optarg;
                break;
            case 'S':
                srcmac = libnet_hex_aton(optarg, &len);
                break;
            case 'D':
                dstmac = libnet_hex_aton(optarg, &len);
                break;
            case 's':
                s = inet_addr(optarg); 
                break;
            case 'd':
                d = inet_addr(optarg);
        break;
            default:
                usage(argv[0]);
                exit(-1);           
            }
    argc -= optind;
    argv += optind;

    l = libnet_init(LIBNET_LINK_ADV, device, errbuf);
    if (l == NULL)
    {
        fprintf(stderr, "%s", errbuf);
        exit(EXIT_FAILURE);
    }

    if (srcmac == NULL)
    {
        srcmac = libnet_hex_aton("de:ad:de:ad:de:ad", &len);
    }

   if (dstmac == NULL) 
    {
        dstmac = libnet_hex_aton("ff:ff:ff:ff:ff:ff", &len);
    }

    if (s == 0) 
    {
        s = libnet_get_ipaddr4(l);
    }

    if (d == 0) 
    {
        d = inet_addr("255.255.255.255");
    }

    t = libnet_build_arp(
            ARPHRD_ETHER,                           /* hardware addr */
            ETHERTYPE_IP,                           /* protocol addr */
            6,                                      /* hardware addr size */
            4,                                      /* protocol addr size */
            type,                                   /* operation type */
            srcmac,                                 /* sender hardware addr */
            (u_int8_t *)&s,                         /* sender protocol addr */
            dstmac,                                 /* target hardware addr */
            (u_int8_t *)&d,                         /* target protocol addr */
            NULL,                                   /* payload */
            0,                                      /* payload size */
            l,                                      /* libnet context */
            0);                                     /* libnet id */
    if (t == -1)
    {
        fprintf(stderr, "Can't build ARP header: %s\n", libnet_geterror(l));
        goto bad;
    }

    t = libnet_autobuild_ethernet(
            dstmac,                                 /* ethernet destination */
            ETHERTYPE_ARP,                          /* protocol type */
            l);                                     /* libnet handle */
    if (t == -1)
    {
        fprintf(stderr, "Can't build ethernet header: %s\n",
                libnet_geterror(l));
        goto bad;
    }

    if (libnet_adv_cull_packet(l, &packet, &packet_s) == -1)
    {
        fprintf(stderr, "%s", libnet_geterror(l));
    }
    else
    {
        fprintf(stderr, "packet size: %d\n", packet_s);
        libnet_adv_free_packet(l, packet);
    }

    c = libnet_write(l);

    if (c == -1)
    {
        fprintf(stderr, "Write error: %s\n", libnet_geterror(l));
        goto bad;
    }
    else
    {
        fprintf(stderr, "Wrote %d byte ARP packet from context \"%s\"; "
                "check the wire.\n", c, libnet_cq_getlabel(l));
    }
    return (EXIT_SUCCESS);
bad:
    libnet_destroy(l);
    return (EXIT_FAILURE);
}

void 
usage(char *progname)
{
    fprintf(stderr, "%s [-R|-r] [-i interface] [-S SourceMAC] [-D DestMAC] [-s SourceIP] [-d DestIP] [-h]\n", progname);
    fprintf(stderr, "\t[-R|-r]       : Request or reply [default]\n");
    fprintf(stderr, "\t[-i interface]: Interface to send packets to\n");
    fprintf(stderr, "\t[-S SourceMAC]: Specify the source MAC address\n");  
    fprintf(stderr, "\t[-D DestMAC]  : Specify the destination MAC address\n");
    fprintf(stderr, "\t[-s SourceIP] : Specify the source IP\n");
    fprintf(stderr, "\t[-d DestIP]   : Specify the destination IP\n");
    fprintf(stderr, "\t[-h]          : This stuff\n");
}
/* EOF */
